/* Minimum functional requirements of
Music Store App and their associated queries.
Where variations are given they are
given in order of increasing complexity */

/* 1a. Retrieve a list of Albums
Include the UPC, title, price, artist name
and genre in the result-set.
Order by title (a-z) */
SELECT Album.title, Album.price, Artist.first_name, Artist.last_name, Genre.name AS genre 
FROM Album /* notice the use of a column alias! */
INNER JOIN Artist
	ON Album.artist_id=Artist.id
INNER JOIN Genre
	ON Album.genre_id=Genre.id
ORDER BY title ASC; /* causes results to be ordered by album title */

/* 1b. Retrieve a list of Albums
Include the UPC, title, price, artist name,
genre, and number of tracks in the result-set.
Include only albums which have tracks in DB.
Order by title (a-z) */
SELECT Album.title, Album.price, Artist.first_name, Artist.last_name, Genre.name AS genre, (SELECT COUNT(*) FROM Track WHERE Album.upc=Track.album_upc) AS num_tracks 
FROM Album /* notice the nested query, aggregate function and aliases! */
INNER JOIN Track
	ON Album.upc=Track.album_upc 
INNER JOIN Artist 
	ON Album.artist_id=Artist.id 
INNER JOIN Genre
	ON Album.genre_id=Genre.id 
ORDER BY title ASC;

/* 1c. Retrieve a list of Albums
Include the UPC, title, price, artist name,
genre, and number of tracks in the result-set.
Include ALL albums, even those without tracks.
Order by title (a-z) */
SELECT Album.title, Album.price, Artist.first_name, Artist.last_name, Genre.name AS genre, (SELECT COUNT(*) FROM Track WHERE Album.upc=Track.album_upc) AS num_tracks
FROM Album /* notice the nested query, aggregate function and aliases! */
LEFT JOIN Track /* LEFT JOIN causes all Albums to be returned, even when no matching record in Track table */
	ON Album.upc=Track.album_upc 
INNER JOIN Artist 
	ON Album.artist_id=Artist.id 
INNER JOIN Genre
	ON Album.genre_id=Genre.id 
ORDER BY title ASC;

/* ------------------------------------ */

/* 2. Retrieve a list of tracks
associated with a specific album */
SELECT t.name AS track, t.track_number, a.title AS album /* notice use of column aliases! */
FROM Track t /* notice use of table aliases! */
INNER JOIN Album a
	ON t.album_upc=a.upc
WHERE a.upc='018274372722' /* EXAMPLE */
ORDER BY track_number ASC;

/* ------------------------------------ */

/* 3. Insert a new track */
INSERT INTO Track (name, album_upc, track_number)
VALUES
('Sunday Coming', '726517237627', 1); /* EXAMPLE */

/* ------------------------------------ */

/* 4. Delete a track */
DELETE FROM Track WHERE album_upc='726517237627'; /* EXAMPLE */

/* ------------------------------------ */

/* 5a. Retrieve details about an order
Include the customer_id, delivery method
and total order value in the result-set */

SELECT Transaction.customer_id, Transaction.delivery, SUM(Album.price*LineItem.quantity) AS order_total /* notice use of aggregate function! */
FROM Transaction 
INNER JOIN LineItem 
	ON Transaction.id=LineItem.trans_id 
INNER JOIN Album 
	ON Album.upc=LineItem.album_upc
WHERE Transaction.id=1; /* EXAMPLE */

/* 5b. Retrieve details about an order
Include the transaction id, customer_id
and the number of items ordered in the 
result-set */
SELECT t.id, t.customer_id, SUM(li.quantity) AS num_items 
FROM Transaction t 
INNER JOIN LineItem li 
	ON t.id=li.trans_id 
GROUP BY t.id; /* group results by transaction id */

/* 5c. Does the same as the previous query
but using a nested query rather than a JOIN */
SELECT t.id, t.customer_id, 
(SELECT SUM(li.quantity) FROM LineItem li 
	WHERE li.trans_id=t.id) AS num_items
FROM Transaction t;
